package org.bsc.maven.plugin.processor.test;

public class TestWikiProcessorClass {

    @ServiceDocumentation("service1")
    public void service1() {


    }

    @ServiceDocumentation("service2")
    public void service2( @ParameterDocumentation("param1") String param1) {


    }
}
